@@ -2,9 +2,16 @@ import numpy as np
 from .constants import *
 
 import itertools
+from functools import reduce
 
 
 
+class pencil:
+    def __init__(self,E,A):
+        self.E = E
+        self.A = A
+
+
 def check_commuting_set(list_of_array):
     '''
     Function to check if all matrices in the argument list pairwise commute.
@@ -239,8 +246,8 @@ def get_random_element_of_group(basis):
         E += c*b
     return E
 
-def get_lie_group(seed):
 
+def get_lie_group(seed):
     """
     A Function to calcute the smallest lie group including all matrices form the seed.
     Parameters: 
@@ -271,4 +278,78 @@ def get_lie_group(seed):
                 current_basis = new_basis
                 current_dim = new_dim
                 dimension_has_changed = True
-    return current_basis
\ No newline at end of file
+    return current_basis
+
+
+def kron_sum(A,B):
+    """
+    An implementation of the Kronecker sum. 
+    [Horn, R. A. and Johnson, C. R. Topics in Matrix Analysis. Cambridge, England: Cambridge University Press, p. 208, 1994.]
+    """
+    n = len(A)
+    I = np.eye(n)
+    return(np.kron(A,I)+np.kron(I,B))
+
+
+def get_commuting_basis(A):
+    """
+    A function to calculate a basis of matrices which commute.
+    """
+    n,m = A.shape
+    S = kron_sum(A,-A.T)
+    r = np.linalg.matrix_rank(S)
+    U,s,V = np.linalg.svd(S,0)
+    N = get_right_null_space(S,r)
+    # Reshape the basis:
+    B = np.reshape(N.T,[n-r,n,m])
+    return(B)
+
+
+def get_random_commuting_matrix(A):
+    """
+    A function to calculate a random matrix that commutes with A.
+    Parameters:
+        A (2d numpy array): A square matrix.
+    Returns:
+        B (2d numpy array): A matrix that commutes with A.
+    """
+    basis = get_commuting_basis(A)
+    s = lambda a,b: a+b
+    B = reduce(s, [np.random.randn()*b for b in basis])
+    return B
+
+
+def J(n ,lam=1):
+    """
+    A function to calculate a Jordan block of size n times n. The diagonal element is lambda.
+    Parameters:
+        n(int): the size of the block
+    Returns:
+        J 2D numpy array: A Jordan block.
+    """
+    J = np.eye(n+1)
+    J = J[1:,0:-1]
+    J[range(n),range(n)] = lam
+    return J
+
+
+def blockdiag(mat_list):
+    """
+    Concatenates a list of matrices into a block diagonal matrix.
+    Parameters:
+        mat_list List(2d array): a list of 2d arrays.
+    Returns:
+        M 2d array: block diagonal matrix.
+    """
+    sh = [list(m.shape) for m in mat_list]
+    l = [list(m.shape) for m in mat_list]
+    tuple_sum = lambda a,b: [i+j for i,j in zip(a,b)]
+    n,m = reduce(tuple_sum ,l)
+    M = np.zeros((n,m))
+    p1 = p2 = 0
+    for m in mat_list:
+        s1,s2 = m.shape
+        M[p1:p1+s1,p2:p2+s2] = m
+        p1 += s1
+        p2 += s2
+    return M
\ No newline at end of file
